def is_not_none(object=None, **kwargs):
    return (kwargs.get('on_true', True) if object is not None else kwargs.get('on_false', False))

def is_none(object=None, **kwargs):
    return (kwargs.get('on_true', True) if object is None else kwargs.get('on_false', False))



import operator

def __apply_function(function, parameter):
    if function.__dict__.get('__args_length__', 1) == 1:
        return function(parameter)
    else:
        return function(*parameter)

def __get_args(function):
    if function is not None:
        function.__dict__['__args_length__'] = len(inspect.getargspec(function).args)

def __evaluate_condition(condition, element, inverse=False):
    if not condition: return not inverse
    return inverse ^ condition(*element)

def __apply_condition(self, **kwargs):
    if kwargs.get('condition'):
        return {key: value for key, value in self.items() \
                if __evaluate_condition(
                    kwargs.get('condition'), (key, value), kwargs.get('inverse', False)
                )}
    return self


def transform(self, key_function=None, value_function=None):
    key_function = key_function if key_function is not None else lambda element: element
    value_function = value_function if value_function is not None else lambda element: element
    __get_args(key_function)
    __get_args(value_function)
    return {__apply_function(key_function, key): __apply_function(value_function, value) \
            for key, value in self.items()}

def keys_list(self):
    return [*self]

def values_list(self):
    return [*self.values()]

def filter(self, condition):
    return __apply_condition(self, condition=condition)

def remove(self, key_function=None, value_function=None, operator='and'):
    operator = operator.and_ if operator == 'and' else operator.or_
    key_function = key_function if key_function is not None else lambda element: True
    value_function = value_function if value_function is not None else lambda element: True
    __get_args(key_function)
    __get_args(value_function)
    return {key: value for key, value in self \
            if operator(__apply_function(key_function, key), __apply_function(value_function, value))}

def reverse(self, keep_duplicate=False):
    if not keep_duplicate:
        return {value: key for key, value in self.items()}
    else:
        reverse_dict, duplicate_value = {}, []
        for key, value in self.items():
            if value in reverse_dict:
                if value in duplicate_value:
                    reverse_dict[value].append(key)
                else:
                    reverse_dict[value] = [reverse_dict[value], key]
            else:
                reverse_dict[value] = key

        return reverse_dict

def deep_merge(self, target_dict, max_depth=None, depth=1):
    merged_dict = self.copy()
    for key, value in target_dict.items():
        if key in merged_dict:
            if isinstance(merged_dict[key], dict) and isinstance(value, dict):
                if max_depth is not None and max_depth == depth:
                    merged_dict[key] = [merged_dict[key], value]
                else:
                    merged_dict[key] = deep_merge(merged_dict[key], value, max_depth, depth + 1)
            else:
                merged_dict[key] = [merged_dict[key], value]
        else:
            merged_dict[key] = value

    return merged_dict



import math
def is_positive(self, **kwargs):
    return (kwargs.get('on_true', True) if self > 0 else kwargs.get('on_false', False))

def is_negetive(self, **kwargs):
    return (kwargs.get('on_true', True) if self < 0 else kwargs.get('on_false', False))

def is_zero(self, **kwargs):
    return (kwargs.get('on_true', True) if self == 0 else kwargs.get('on_false', False))

def safe_divide(self, denominator, default=math.nan):
    return (default if denominator == 0 else self / denominator)



import operator, collections, inspect

def __is_type(obj, target_obj):
    if target_obj == 'iterable': return type(obj).__name__ in ['list', 'set', 'tuple', 'dict']
    if type(target_obj).__name__ == 'str': return type(obj).__name__ == target_obj
    if type(target_obj).__name__ == 'list': return type(obj).__name__ in target_obj
    return False

def __apply_function(function, parameter):
    if function.__dict__.get('__args_length__', 1) == 1:
        return function(parameter)
    else:
        return function(*parameter)

def __get_args(function):
    if function is not None:
        function.__dict__['__args_length__'] = len(inspect.getargspec(function).args)

def __evaluate_condition(condition, element, inverse=False):
    if condition is None: return not inverse
    return inverse ^ __apply_function(condition, element)

def __apply_condition_cast(self, **kwargs):
    __get_args(kwargs.get('condition'))
    if kwargs.get('condition') or not kwargs.get('cast'):
        return [kwargs.get('cast')(element) if kwargs.get('cast') else element \
                for element in self \
                if __evaluate_condition(kwargs.get('condition'), element, kwargs.get('inverse', False))]
    else:
        return self



def unique(self):
    return [*{*self}]

def to_set(self):
    return {*self}

def to_tuple(self):
    return (*self, )

def to_dict(self, key_function, value_function):
    __get_args(key_function)
    __get_args(value_function)
    return {__apply_function(key_function, element): __apply_function(value_function, element) for element in self}

def remove_index(self, index=[], **kwargs):
    if index == [] and not kwargs.get('condition'):
        return self
    elif isinstance(index, int):
        index = [index]

    conditional_index = __apply_condition_cast(
        [*range(len(self))], condition=kwargs.get('condition')
    ) if kwargs.get('condition') else []
    return __apply_condition_cast(
        [element for index_value, element in enumerate(self) \
         if index_value not in index + conditional_index],
        cast=kwargs.get('cast')
    )

def remove_values(self, values=[], **kwargs):
    if values == [] and not kwargs.get('condition'):
        return self
    elif values == [] and kwargs.get('condition'):
        return __apply_condition_cast(self, inverse=True, **kwargs)

    return __apply_condition_cast(
        [element for element in self if element not in values], inverse=True, **kwargs
    )

def filter(self, condition, **kwargs):
    return __apply_condition_cast(self, condition=condition, cast=kwargs.get('cast'))

def compact(self):
    compact_condition = lambda element: element is not None or (__is_type(element, 'iterable') and any(element))
    return __apply_condition_cast(self, condition=compact_condition)

def transform(self, element_function):
    return [__apply_function(element_function, element) for element in self]

def count_all(self):
    return {**collections.Counter(self)}

def intersection(self, target_object):
    return [*({*self}.intersection({*target_object}))]

def flatten(self):
    flatten_list = []
    for element in self:
        if isinstance(element, list):
            flatten_list.extend(flatten(element))
        elif isinstance(element, tuple) or isinstance(element, set):
            flatten_list.extend(flatten([*element]))
        else:
            flatten_list.append(element)

    return flatten_list

def get_index(self, index, default_value=None):
    try:
        return self[index]
    except IndexError:
        return default_value

def execute(self, execute_function):
    __get_args(execute_function)
    for element in self:
        __apply_function(execute_function, element)

    return self



import py_toolbelt.list_toolbelt as ltb
import py_toolbelt.dict_toolbelt as dtb
import py_toolbelt.other_toolbelt as otb
import py_toolbelt.int_toolbelt as itb
import py_toolbelt.float_toolbelt as ftb
import py_toolbelt.set_toolbelt as stb
import py_toolbelt.tuple_toolbelt as ttb
from forbiddenfruit import curse, reverse
import inspect

DATA_TYPE_MODULE_MAP = {list: ltb, dict: dtb, int: itb, float: ftb, set: stb, tuple: ttb}

def activate():
    for data_type, module in DATA_TYPE_MODULE_MAP.items():
        for function_name, function in inspect.getmembers(module, inspect.isfunction):
            if not function_name.startswith('__'):
                curse(data_type, function_name, function)

    for data_type in [set, list, str, dict, tuple, int, float, type(None)]:
        curse(data_type, 'is_none', otb.is_none)
        curse(data_type, 'is_not_none', otb.is_not_none)

def deactivate():
    for data_type, module in DATA_TYPE_MODULE_MAP.items():
        for function_name, function in inspect.getmembers(module, inspect.isfunction):
            if not function_name.startswith('__'):
                reverse(data_type, function_name)

    for data_type in [set, list, str, dict, tuple, int, float, type(None)]:
        reverse(data_type, 'is_none')
        reverse(data_type, 'is_not_none')



from setuptools import setup, find_packages

VERSION = '0.2.1'
DESCRIPTION = 'PyToolBelt - Extend your built-in methods.'
LONG_DESCRIPTION = open('README.md', 'r').read()

# Setting up
setup(
       # the name must match the folder name 'verysimplemodule'
        name="py_toolbelt",
        version=VERSION,
        author="Soami Charan",
        author_email="charansoami@gmail.com",
        description=DESCRIPTION,
        long_description=LONG_DESCRIPTION,
        long_description_content_type='text/markdown',
        license='MIT',
        packages=find_packages(),
        install_requires=['forbiddenfruit'],
        keywords=['python', 'PyToolBelt'],
        classifiers= [
            "Development Status :: 3 - Alpha",
            "Intended Audience :: Education",
            "Programming Language :: Python :: 3",
        ]
)
